;; -*- lexical-binding: t -*-

(require 'align)

(defun test/contrast (fg bg palette)
  (let* ((fg-rgb (car (alist-get fg palette)))
         (bg-rgb (car (alist-get bg palette)))
         (face `(:foreground ,fg-rgb :background ,bg-rgb :extend t))
         (ratio (modus-themes-contrast fg-rgb bg-rgb))
         (ratio-face (append face
                             '(:weight bold)
                             (when (< ratio 7.0)
                               '(:inverse-video t)))))
    (insert
     (propertize (format "%s/%s %s/%s " fg bg fg-rgb bg-rgb)
                 'face face)
     (propertize (format "%.3f" ratio) 'face ratio-face)
     (propertize "\n" 'face face))))

(defun test/analyze (theme)
  (load-theme theme t)
  (let* ((palette (symbol-value
                   (intern-soft (format "%s-palette" theme))))
         (text-buffer (get-buffer-create
                       (format "*%s*" theme))))
    (pop-to-buffer text-buffer)
    (delete-other-windows)
    (insert (format "%s\n" theme))
    (pcase-dolist (`(,c1 ,c2)
                   (list
                    ;; magit-revision
                    ;;; magit-diff-removed
                    '(fg-removed bg-removed-faint)
                    ;;; magit-diff-removed-highlight
                    '(fg-removed bg-removed)
                    ;;; magit-diff-removed-highlight ∪ diff-refine-removed
                    '(fg-removed bg-removed-refine)
                    ;;; magit-diff-added
                    '(fg-added bg-added-faint)
                    ;;; magit-diff-added-highlight
                    '(fg-added bg-added)
                    ;;; magit-diff-added-highlight ∪ diff-refine-added
                    '(fg-added bg-added-refine)
                    ;; ediff
                    ;;; ediff-current-diff-A
                    '(fg-main bg-removed)
                    ;;; ediff-fine-diff-A
                    '(fg-main bg-removed-refine)
                    ;;; ediff-fine-diff-A ∪ font-lock-comment-face
                    '(fg-dim bg-removed-refine)
                    ;;; ediff-current-diff-B
                    '(fg-main bg-added)
                    ;;; ediff-fine-diff-B
                    '(fg-main bg-added-refine)
                    ;;; ediff-fine-diff-B ∪ font-lock-comment-face
                    '(fg-dim bg-added-refine)))
      (test/contrast c1 c2 palette))
    (let ((align-rules-list
           '((columns (regexp . "\\(^\\|\\S-\\)\\([ 	]+\\)\\(\\S-\\|$\\)")
                  (group . 2)
                  (modes . '(fundamental-mode))
                  (repeat . t))
             (ratio (regexp . "\\(\\s-+[0-9]+\\)\\.")
                    (modes . '(fundamental-mode))
                    (justify . t)))))
      (align (point-min) (point-max)))
    (message nil)
    (let ((png-data (x-export-frames nil 'png))
          (png-buffer (find-file (format "/tmp/%s.png" theme))))
      (insert png-data)
      (save-buffer))))

(dolist (theme '(modus-operandi
                 modus-operandi-tinted
                 modus-operandi-deuteranopia
                 modus-vivendi
                 modus-vivendi-tinted
                 modus-vivendi-deuteranopia))
  (test/analyze theme))
